;;; -*- Mode:Asm Mode:outline-minor-mode outline-regexp:";;;+" comment-start: "; " -*-
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; equates-lpcxpresso.asm

;;; This section applies to the LPCXpresso LPC1769 board

        ;; The LED is on Port 0, bit 22
        .equ LED_MASK, BIT22

;;; NOTE
        ;; Four subroutines and 2 Forth primitives must be defined.

        ;; The following subroutines must be defined, even if they do nothing,
        ;; because riscy.asm will call them.
        ;; 
        ;;      setup_clocks
        ;;      setup_ports
        ;;      ledOnSub
        ;;      ledOffSub

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        ;.equ PLLCLKIN, 4000000   ;  Internal RC clock frequency
        .equ PLLCLKIN, 12000000   ;  Main crystal clock frequency
        .equ PLL_MULTIPLIER, 1   ;  Multiplier must be 1 since we turn off PLL
        .equ CPUDIVISOR, 1       ;  (does the LPC17xx have a cpu clock divisor?)
        .equ PCLKDIVISOR, 4      ; must be 1, 2, or 4
        .equ TIMER0_PRESCALE_DIVISOR, 1
        .equ BAUDRATE, 38400
        .equ SPIDIVISOR, 128     ; slow it way down for testing


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Equates that do calculations and so are not likely to change
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; frequency is given in number of clocks per second

        ;; Unlike the LPC23xx and maybe others, the LPC2103 has a single
        ;; PCLK.
        
        .equ PLLCLK, (PLLCLKIN * PLL_MULTIPLIER)
        .equ CCLK, (PLLCLK / CPUDIVISOR)

        ;; We do not need to use the "case" statement; we could simply
        ;; do     .equ PCLKVALUE, (PCLKVALUE % 4)   instead
        .if PCLKDIVISOR == 1
          .equ PCLKVALUE, 1             ; divide by 1
        .endif
        .if PCLKDIVISOR == 2
          .equ PCLKVALUE, 2             ; divide by 2
        .endif
        .if PCLKDIVISOR == 4
          .equ PCLKVALUE, 0             ; divide by 4
        .endif

        .equ  PCLK, CCLK / PCLKDIVISOR   ; most timing depends on PCLK
        
        .equ PCLK_TIMER0_DIVISOR, PCLKDIVISOR
        
        .equ TIMER0FREQ, ((CCLK / PCLK_TIMER0_DIVISOR) / TIMER0_PRESCALE_DIVISOR)
        
        .equ  SPICLK, PCLK / SPIDIVISOR

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Clocks
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
setup_clocks:

        ; Switch to the external 12 MHz clock (the serial port would
        ; not run reliably at 38400 BPS using the internal 4 MHz RC
        ; clock).

        str lr, [sp, #-4]!  ; push return address onto machine stack
                            ;  because this subroutine will itself call
                            ;  other subroutines

        ; Turn on the main clock, wait for it to stabilize, then
        ; switch from the IRC clock to the main clock.

        ldr r1, = SCS
        ldr r0, [r1]
        bic r0, r0, #0x10     ; clear bit 4 to select 1-20MHz OSRANGE
        str r0, [r1]
        orr r0, r0, #0x20  ; set bit 5 OSCEN to start main oscillator
        str r0, [r1]
oscstab:   
        ; wait for main oscillator to stabilize by reading bit 6 (OSCSTAT)
        ; until it goes to 1.
        ldr r0, [r1]
        tst r0, #0x40       ; Wait for OSCSTAT
        beq oscstab         ;   to go true.

chgclk:
        ; Now that the main oscillator is stable,
        ; switch from the IRC clock to the main clock
        ldr r1, = CLKSRCSEL
        mov r0, #1
        strb r0, [r1]

        ldr pc, [sp], #4    ; rts


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; I/O Ports and Peripherals
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
setup_ports:

        ;; Clear all the mask bits so that no pins are masked
        ldr r6, = FIO0MASK
        mov r0, #0
        str r0, [r6]            ; FIO0MASK

        ;; set all pins to be GPIO then later OR in any needed 1 bits
        mov r0, #0
        ldr r6, = PINSEL0
        str r0, [r6]
        ldr r6, = PINSEL1
        str r0, [r6]
        
        ;; Set up LED pin (this assumes the LPCXpresso LPC1769 board).
        ldr r6, = PINSEL1
        ;; set P0.22 as a GPIO pin
        ;; P0.22 is controlled by bits 13:12 of PINSEL1
        ldr r0, [r6]
        bic r0, r0, # 0x00003000     ; clear bits 13:12 to force GPIO mode
        str r0, [r6]
        
        ;; make the pin connected to the LED be an output, all others will be input pins.
        ldr r6, = FIO0DIR
        ldr r0, = LED_MASK       ; turn on the LED bit
        str r0, [r6]             ; make bit the LED bit be an output and all others be inputs


        ;; UART0 Initialization
        ;; on the LPC17xx, P0.2 is TXD0 (Tx for UART0)
        ;;                 P0.3 is RXD0 (Rx for UART0)
        ;; Transmit on P0.2 (TXD0), Receive on P0.3 (RXD0)
        ;; we do not need to set TXD0 as an output, right?  Right!
pin_init:    
        ldr r6, = PINSEL0
        ldr r0, [r6]
        orr r0, r0, # 0x50   ; bits 7:6 control RXD0, bits 5:4 control TXD0  (0101 0000)
                             ;   we set each pair to '01' to select the UART function
        str r0, [r6]

setup_uarts:
        ldr r6, = U0BASE 
        
        ; Write to U0LCR to set DLAB bit and then set baud rate, etc.
        mov r1, #0x83        ; access divisor latches, 8 bits, 1 stop, no parity
        strb r1, [r6, # ULCR]

        ;; see equates-lpc17xx.asm for formula

        ;; 38400 BPS with PCLK = 12 MHz
        mov r1, # 3
        strb r1, [r6, # UDLL]
        mov r1, # 0
        strb r1, [r6, # UDLM]

        ; following works for 4 MHz internal or 12 MHz external for various baud rates
        ;; Set fractional divider register
        ;;   bits 7:4 MulVal = 8
        ;;   bits 3:0 DivAddVal = 5
        ;;   FDR := '1000 0101' 
        mov r1, # 0x85
        strb r1, [r6, # UFDR]

        mov r1, #03
        strb r1, [r6, # ULCR]   ; turn off access to divisor latches but
                                ;   leave uart0 set to 8 bits, 1 stop, no parity

        ;mov r0, #1
        mov r0, # 0x81          ;  bits 7:6 = '10' for a Rx trigger level of 8 characters
                                ;   (even though we are not using interrupt or DMA at this time)
        strb r0, [r6, # UFCR]   ;  enable FIFOs (must set bit 0 for UART to work at all)

        mov pc, lr              ; ret

        
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; LED Subroutines
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;;  Note, these routines are specific to a board which has
        ;;  an LED on Port0,
        ;;    (P0.22 in the case of the LPCXpresso LPC1769 board, high = on).
ledOnSub:     
        ldr r6, = FIO0SET
        ldr r1, = LED_MASK      ; LED is connected to this bit of Port 0
        str r1, [r6]            ; make LED bit one (to turn on the LED)
        mov pc, lr
ledOffSub:     
        ldr r6, = FIO0CLR
        ldr r1, = LED_MASK      ; LED is connected to this bit of Port 0
        str r1, [r6]            ; make LED bit zero (to turn off the LED)
        mov pc, lr

;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Subroutines Used Only By Other Subroutines In This File
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Feed the PLL
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;; This or something similar might be needed if we use the PLL
;; feed_pll:  ;; PLL feed sequence
;;         ldr r1, = PLL0FEED
;;         mov r0, #0xaa
;;         strb r0, [r1]
;;         mov r0, #0x55
;;         strb r0, [r1]
;;         mov pc, lr
